grammar LossGrammar;

// [Execution of ANTLRWorks]
// $ java -jar /usr/local/antlrworks/dist/antlrworks-1.4.3.jar

// [Remark about eclipse test execution]
// as the compilation is done in command line: 
//    java org.antlr.Tool LossGrammar.g  -verbose -o /home/def/Workspaces/eclipse/default/JPLCore/src/org/expasy/jpl/core/mol/polymer/pept/rule
// and generates  LiberLexer.java and LiberParser.java
// these code file have to be refreshed for eclipse to reload the updated .class

// [Input example]
// K/20:450 -> -34.9

@header {
package org.expasy.jpl.core.mol.polymer.pept.rule;

import java.util.ArrayList;
import java.util.Collection;

import org.expasy.jpl.commons.collection.JPLInterval;
import org.expasy.jpl.core.mol.modif.JPLModification;
import org.expasy.jpl.core.mol.polymer.pept.matcher.JPLAAMotifMatcher;
import org.expasy.jpl.core.mol.polymer.pept.rule.JPLEditionRule;
import org.expasy.jpl.core.mol.polymer.pept.rule.JPLEditionRule.EditionAction;
import org.expasy.jpl.core.mol.polymer.pept.rule.JPLEditionRule.EditionAction.AtomicAction;

}

@lexer::header {
package  org.expasy.jpl.core.mol.polymer.pept.rule;
}

@members {
List<JPLEditionRule> rules = new ArrayList<JPLEditionRule>();
}

// Grammar
rules : 
	rule (';' rule )*
	;

rule:
	target '->'  actions  {
		StringBuilder name = new StringBuilder();
		name.append("Rule-");
		name.append(rules.size());
		JPLEditionRule rule = JPLEditionRule.newInstance(name.toString(), $target.target, $actions.list);
		
		 rules.add(rule);
	}
	;

target returns [JPLAAMotifMatcher target]:
	aaclass ('/' range) ?  {
		// AA target
		$target = JPLAAMotifMatcher.newInstance($aaclass.aas);
		
		// target a modif ?
		if ($range.to > 0) {
			$target.matchAAWithModifInInterval(
				new JPLInterval.Builder($range.from, $range.to).build());
		 }
	}
	;

// alternative actions
actions returns [List<EditionAction> list]
@init {
$list = new ArrayList<EditionAction>();
}
	:
	a1=action { $list.add($a1.ea); }  (',' a2=action { $list.add($a2.ea); })*
	;
	
action returns [EditionAction ea]
scope {
List<AtomicAction> list;
}
@init {
// initialize symbol list
$action::list = new ArrayList<AtomicAction>();
}
@after {
$ea = EditionAction.newVariableSequentialAction($action::list);
}
	:
	modifaction 	{
		$action::list.add($modifaction.action);
	 }
	| r=removalaction { $action::list.add($r.action); } (m=modifaction { $action::list.add($m.action);  })*
           	;

modifaction returns [AtomicAction action]
@init {
double d = 0;
}
@after {
$action = AtomicAction.newModifAction(JPLModification.valueOf(d));
}
	:
	'-' INT	{
		d = Double.parseDouble("-"+$INT.text);
	}
	|'-' FLOAT	{
		d = Double.parseDouble("-"+$FLOAT.text);
	}
	;

removalaction returns [AtomicAction action]:
	'del'  {
		$action = AtomicAction.newRemovalAction();
	}
	;


aaclass returns [String aas]
@init{
StringBuilder sb = new StringBuilder();
}

@after{
 $aas=sb.toString();
}
	:
	AA { sb.append($AA.text);  }
	| '['  a=AA { sb.append("["); sb.append($a.text); } 
	      (b=AA { sb.append($b.text);  } )* 
	    ']'  { sb.append("]"); }
	;

range returns [double from, double to]:
	f=number  ':' t= number  {
		$from = $f.value; 
		$to = $t.value;
	}
	;

number returns [double value]:
	INT {  
		$value = Integer.parseInt($INT.text); 
	}
	| FLOAT {
		$value = Double.parseDouble($FLOAT.text);
	}
	;

// Lexical level
INT:   '0'..'9'+ ;
FLOAT: INT ('.' INT);
AA:   'A'..'Z' ;
WS  :   (' '|'\t')+ {skip();} ;
